package handler

import (
	"fmt"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/error_process"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/model"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/service"
	bgm "git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/util"
	"github.com/gofiber/fiber/v2"
	ctxLogger "github.com/luizsuper/ctxLoggers"
	"go.uber.org/zap"
	"strconv"
)

type (
	CategoryReq struct {
		Category *model.Category `json:"category" `
	}
	CategoriesTagReq struct {
		Relation *model.CategoriesTagDto `json:"relation" `
	}
)

func GetCategoryH(ctx *fiber.Ctx) error {
	page := ctx.Query("page", "1")
	limit := ctx.Query("size", "10")
	iPage, _ := strconv.Atoi(page)
	iLimit, _ := strconv.Atoi(limit)
	s := ctx.Query("query", "")
	queryMap := bgm.GetQueryMap(s)
	m, num, err := service.GetCategory(iPage, iLimit, queryMap, ctx)
	if err != nil {
		ctxLogger.FError(ctx, error_process.ServiceErr, zap.String("", fmt.Sprintf("%+v", err)))
		return ctx.JSON(fail(nil))
	}
	return ctx.JSON(success(&Data{
		Elements: m,
		Total:    num,
	}))
}

//CreateCategoryH 创建类别
func CreateCategoryH(ctx *fiber.Ctx) error {
	m := new(CategoryReq)
	if err := createCategoryParamCheck(m, ctx); err != nil {
		ctxLogger.FError(ctx, error_process.BodyParserErr, zap.String("", fmt.Sprintf("%v", err)))
		return ctx.JSON(fail(nil))
	}
	err := service.CreateCategory(m.Category, ctx)
	if err != nil {
		ctxLogger.FError(ctx, error_process.ServiceErr, zap.String("", fmt.Sprintf("%+v", err)))
		return ctx.JSON(fail(nil))
	}
	return ctx.JSON(success(nil))
}

func UpdateCategoryH(ctx *fiber.Ctx) error {
	m := new(CategoryReq)
	if err := checkParam(m, ctx); err != nil {
		ctxLogger.FError(ctx, error_process.BodyParserErr, zap.String("", fmt.Sprintf("%v", err)))
		return ctx.JSON(fail(nil))
	}
	err := service.UpdateCategory(m.Category, ctx)
	if err != nil {
		ctxLogger.FError(ctx, error_process.ServiceErr, zap.String("", fmt.Sprintf("%+v", err)))
		return ctx.JSON(fail(nil))
	}
	return ctx.JSON(success(nil))
}

func DeleteCategoryH(ctx *fiber.Ctx) error {
	m := new(CategoryReq)
	if err := checkParam(m, ctx); err != nil {
		ctxLogger.FError(ctx, error_process.BodyParserErr, zap.String("", fmt.Sprintf("%v", err)))
		return ctx.JSON(fail(nil))
	}
	err := service.DeleteCategory(m.Category, ctx)
	if err != nil {
		ctxLogger.FError(ctx, error_process.ServiceErr, zap.String("", fmt.Sprintf("%+v", err)))
		return ctx.JSON(fail(nil))
	}
	return ctx.JSON(success(nil))
}

// CreateTagForCategory 为Category 新增tag
func CreateTagForCategory(ctx *fiber.Ctx) error {
	m := new(CategoriesTagReq)
	if err := checkParam(m, ctx); err != nil {
		ctxLogger.FError(ctx, error_process.BodyParserErr, zap.String("", fmt.Sprintf("%v", err)))
		return ctx.JSON(fail(nil))
	}
	err := service.CreateTagForCategory(m.Relation, ctx)
	if err != nil {
		ctxLogger.FError(ctx, error_process.ServiceErr, zap.String("", fmt.Sprintf("%+v", err)))
		return ctx.JSON(fail(nil))
	}
	return ctx.JSON(success(nil))
}

// DeleteTagForCategory 为Category 删除tag
func DeleteTagForCategory(ctx *fiber.Ctx) error {
	m := new(CategoriesTagReq)
	if err := ctx.BodyParser(m); err != nil {
		ctxLogger.FError(ctx, error_process.BodyParserErr, zap.String("", fmt.Sprintf("%+v", err)))
		return ctx.JSON(fail(nil))
	}
	err := service.DeleteTagForCategory(m.Relation, ctx)
	if err != nil {
		ctxLogger.FError(ctx, error_process.ServiceErr, zap.String("", fmt.Sprintf("%+v", err)))
		return ctx.JSON(fail(nil))
	}
	return ctx.JSON(success(nil))
}
