package model

import (
	"fmt"
	"git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/error_process"
	bgm "git.icechen.cn/monorepo/backend/app/brahma/api/murder/internal/util"
	"github.com/pkg/errors"
	"github.com/spf13/cast"
	"gorm.io/gorm"
)

type (
	Tag struct {
		Value string      `json:"value" gorm:"column:value"`
		Uuid  int         `json:"uuid" gorm:"primary_key" valid:"no_empty"`
		IsDel int         `json:"-" gorm:"column:is_del" value:"1|0"`
		Cs    *[]Category `json:"categories,omitempty" gorm:"many2many:c_tag;foreignKey:Uuid;joinForeignKey:TagUid;References:Uuid;JoinReferences:CategoryUid" valid:"no_empty"`
	}
	// TagCategoriesDto 为标签新增分组
	TagCategoriesDto struct {
		CIds []string `json:"category_ids" valid:"no_empty"`
		Tid  int      `json:"tag_id" valid:"no_empty"`
	}
)

func (m *Tag) TableName() string {
	return "tag"
}

func CreateTagM(tag *Tag, db *gorm.DB) error {
	if err := db.Create(tag).Error; err != nil {
		return error_process.CreateErr(err, tag)
	}
	return nil
}

func UpdateTagM(tag *Tag, db *gorm.DB) error {
	if num := db.Model(Tag{}).Where("uuid = ?", tag.Uuid).Updates(*tag).RowsAffected; num != 1 {
		return error_process.UpdateErr(tag, fmt.Sprintf("uuid:%v", tag.Uuid))
	}
	return nil
}

func GetTagM(page, size int, query bgm.QueryMap, db *gorm.DB) ([]Tag, int64, error) {
	t := new([]Tag)
	num := int64(0)
	for k, v := range query {
		db = db.Where(fmt.Sprintf("%v = ?", k), v)
	}
	if num = db.Find(t).RowsAffected; num <= 0 {
		return nil, 0, error_process.GetErr(errors.New(error_process.GormError), query)
	}
	if size > 0 {
		db = db.Limit(size).Offset((page - 1) * size)
	}
	if err := db.Preload("Cs").Find(t).Error; err != nil {
		return nil, 0, err
	}
	return *t, num, nil
}

func CreateCategoryForTagM(tid int, cIds []string, db *gorm.DB) error {
	t := new(Tag)
	//step1 getTag
	if err := db.Where(&Tag{
		Uuid: tid,
	}).Find(t).Error; err != nil {
		return error_process.GetErr(err, t)
	}
	//step2 get categories and check num is expected
	var cs []Category
	if num := db.Where(cIds).Find(&cs).RowsAffected; num != cast.ToInt64(len(cIds)) {
		return error_process.RecordNotMatchWithUniqueErr(cIds, len(cIds), cast.ToInt(num))
	}
	//step3 add association
	if err := db.Model(t).Association("Cs").Append(cs); err != nil {
		return error_process.CommonErr(err)
	}
	return nil
}

func DelCategoryForTagM(tid int, cIds []string, db *gorm.DB) error {
	t := new(Tag)
	//step1 checkTagNum
	if err := db.Where(&Tag{
		Uuid: tid,
	}).Find(t).Error; err != nil {
		return error_process.GetErr(err, t)
	}
	//step2 check cIdsNum
	var cs []Category
	if num := db.Where(cIds).Find(&cs).RowsAffected; num != cast.ToInt64(len(cs)) {
		return error_process.RecordNotMatchWithUniqueErr(cIds, len(cIds), cast.ToInt(num))
	}
	//step3 add association
	err := db.Model(t).Association("Cs").Delete(cs)
	if err != nil {
		return error_process.CommonErr(err)
	}
	return nil
}
